/*
tmpbt.c Version 0.1.0  Btree build test database
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* Usage: tmpbt filename */

#include <bthash.h>

#define MAXLMT 999999

void putstx(char *pgm)
   {
   fprintf(stderr,"Usage:   %s "
      "#records\n", pgm);
   fprintf(stderr,"Example: %s 100\n", pgm);
   fprintf(stderr,"#records is 1 - %d\n",
      MAXLMT);
   exit(1);
   } /* putstx */

int main(int argc, char **argv)
   {
   int opnerr;
   int i;
   int rslt;
   int lmt;
   unsigned char *rndSeed;
   btreefmt *tmpbt;
   char *p,*q;
   unsigned char str[4096];
   if (argc != 2) putstx(*argv);
   lmt = (int) atoi(*(argv+1));
   if (lmt < 1 || lmt > MAXLMT) putstx(*argv);

   rndSeed = (unsigned char *) rndinit();
   if (rndSeed == NULL)
      {
      fprintf(stderr,"tmpbt: rndinit error\n");
      exit(1);
      } /* rndinit err */

   /* btopent numeric parameters */
   /* nodesz, rcdlen, keylen, keyofst, dletofst */
   tmpbt = (btreefmt *) btopent(11,16,8,4,0,&opnerr,rndSeed);
   if (tmpbt == NULL)
      {
      fprintf(stderr,"tmpbt: open "
	 "error %d\n", opnerr);
      free(rndSeed);
      exit(1);
      } /* open error */
   /* process starts here */

   rslt = 0;
   i = lmt;
   while (!rslt && (i--))
      {
      p = (char *) str;
      q = (char *) p + tmpbt->keyofst;
      while (p < q)
	 {
	 *p++ = rndnum(26,rndSeed) + 'a';
	 } /* for each char in rcd */
      q = (char *) p + tmpbt->keylen;
      while (p < q)
	 {
         *p++ = rndnum(26,rndSeed) + 'A';
	 } /* for each char in rcd */
      q = (char *) str + tmpbt->rcdlen;
      while (p < q)
	 {
         *p++ = rndnum(26,rndSeed) + 'a';
	 } /* for each char in rcd */
      *p = '\0';
      rslt = btisrt(tmpbt,ROOT,str);
      if (rslt)
	 {
	 fprintf(stderr,"tmpbt: after btisrt rslt %d\n",
	    rslt);
	 exit(1);
	 } /* btisrt err */
      } /* insert loop */
   rslt = btdbck(tmpbt);
   if (rslt || tmpbt->reason)
      {
      fprintf(stderr,"tmpbt: btdbck error "
	 "rslt %d reason %d\n",
         rslt, tmpbt->reason);
      exit(1);
      } /* if btdbck err */
   printf("tmpbt: temporary database %s\n",
      tmpbt->dbname);
   tmpbt->seq = 0;
   bttravrs(tmpbt,ROOT);

   /**********************************************************/
   /* piping the traverse will prevent the */
   /* temporary file from being removed */
   btcloset(tmpbt);
   return(0);
   } /* main */
